@@ -83,10 +83,7 @@ sub is_multipart {undef}
 
 sub is_parsing_body { (shift->{state} // '') eq 'body' }
 
-sub leftovers {
-  my $self = shift;
-  return ($self->{pre_buffer} // '') . ($self->{buffer} // '');
-}
+sub leftovers { shift->{buffer} }
 
 sub parse {
   my $self = shift;
@@ -96,21 +93,6 @@ sub parse {
   return $self if $self->{state} eq 'headers';
   $self->_body;
 
-  # No content
-  if ($self->skip_body) {
-    $self->{state} = 'finished';
-    return $self;
-  }
-
-  # Relaxed parsing
-  my $headers = $self->headers;
-  if ($self->auto_relax) {
-    my $connection = $headers->connection || '';
-    my $len = $headers->content_length // '';
-    $self->relaxed(1)
-      if !length $len && ($connection =~ /close/i || $headers->content_type);
-  }
-
   # Parse chunked content
   $self->{real_size} //= 0;
   if ($self->is_chunked && $self->{state} ne 'headers') {
@@ -127,6 +109,21 @@ sub parse {
     $self->{pre_buffer} = '';
   }
 
+  # No content
+  if ($self->skip_body) {
+    $self->{state} = 'finished';
+    return $self;
+  }
+
+  # Relaxed parsing
+  my $headers = $self->headers;
+  if ($self->auto_relax) {
+    my $connection = $headers->connection || '';
+    my $len = $headers->content_length // '';
+    $self->relaxed(1)
+      if !length $len && ($connection =~ /close/i || $headers->content_type);
+  }
+
   # Chunked or relaxed content
   if ($self->is_chunked || $self->relaxed) {
     $self->{size} += length($self->{buffer} //= '');
